@@ -405,6 +405,7 @@ DECL_PLIST(zip);
 DECL_PLIST(cbtrace);
 DECL_PLIST(wrap);
 DECL_PLIST(event);
+DECL_PLIST(transaction);
 
 void
 initBuildIns(void)
@@ -475,6 +476,7 @@ initBuildIns(void)
   REG_PLIST(cbtrace);
   REG_PLIST(wrap);
   REG_PLIST(event);
+  REG_PLIST(transaction);
 
 #define LOOKUPPROC(name) \
 	{ GD->procedures.name = lookupProcedure(FUNCTOR_ ## name, m); \
