@@ -112,12 +112,8 @@ struct PL_global_data
     int		sig_alert;		/* our alert signal */
 #endif
   } signals;
-#ifdef O_LOGICAL_UPDATE
-  volatile ggen_t _generation;		/* generation of the database */
-#ifdef ATOMIC_GENERATION_HACK
-  volatile gen_t _last_generation;	/* see pl-inline.h, global_generation() */
-#endif
-#endif
+
+  volatile gen_t _generation;		/* generation of the database */
 
   struct
   { int		os_argc;		/* main(int argc, char **argv) */
@@ -713,6 +709,17 @@ struct PL_local_data
   } thread;
 #endif
 
+  struct
+  { gen_t	gen_start;		/* Global start generation */
+    gen_t	gen_base;		/* Local  start generation */
+    gen_t	gen_max;		/* Transaction max gen */
+    gen_t	gen_nest;		/* Start of nested generation */
+    gen_t	generation;		/* Local current generation */
+    Table	clauses;		/* Affected clauses */
+    term_t	id;			/* Default the goal */
+    struct tr_stack *stack;		/* Nested transaction stack */
+  } transaction;
+
 #ifdef O_LOCALE
   struct
   { PL_locale *current;			/* Current locale */
