@@ -988,6 +988,10 @@ Macros for environment frames (local stack frames)
 				      (f)->predicate->functor->arity : \
 				      (f)->clause->clause->prolog_vars)
 
+
+		 /*******************************
+		 *	 LOGICAL UPDATE		*
+		 *******************************/
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Generations must be 64-bit to  avoid   overflow  in realistic scenarios.
 This makes them the only 64-bit value in struct localFrame. Stack frames
@@ -1003,11 +1007,15 @@ We enable this  if the alignment  of an int64_t type  is not the same as
 the alignment of pointers.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-#ifdef O_LOGICAL_UPDATE
 typedef uint64_t gen_t;
 
-#define GEN_MAX (~(gen_t)0)
-#define GEN_NEW_DIRTY (gen_t)0
+#define GEN_INVALID   ((gen_t)0)
+#define GEN_INFINITE  (~(gen_t)0)
+#define GEN_NEW_DIRTY ((gen_t)0)
+#define GEN_TRANSACTION_BASE 0x8000000000000000
+#define GEN_TRANSACTION_SIZE 0x0000000100000000
+#define GEN_MAX (GEN_TRANSACTION_BASE-1)
+#define GEN_TRMAX(g0) ((g0)|(GEN_TRANSACTION_SIZE-1))
 
 #if ALIGNOF_INT64_T != ALIGNOF_VOIDP
 typedef struct lgen_t
@@ -1028,19 +1036,6 @@ typedef uint64_t lgen_t;
 #define setGenerationFrameVal(f, gen) \
 	do { (f)->generation = (gen); } while(0)
 #endif
-#if defined(HAVE_GCC_ATOMIC_8) || SIZEOF_VOIDP == 8
-typedef uint64_t ggen_t;
-#else
-#define ATOMIC_GENERATION_HACK 1
-typedef struct ggen_t
-{ uint32_t	gen_l;
-  uint32_t	gen_u;
-} ggen_t;
-#endif /*HAVE_GCC_ATOMIC_8 || SIZEOF_VOIDP == 8*/
-#else /*O_LOGICAL_UPDATE*/
-#define global_generation()	 (0)
-#define next_global_generation() (0)
-#endif /*O_LOGICAL_UPDATE*/
 
 #define setGenerationFrame(fr) setGenerationFrame__LD((fr) PASS_LD)
 
@@ -1070,6 +1065,10 @@ introduce a garbage collector (TBD).
 #define leaveDefinition(def) (void)0
 
 
+		 /*******************************
+		 *	 INHIBIT SIGNALS	*
+		 *******************************/
+
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 At times an abort is not allowed because the heap  is  inconsistent  the
 programmer  should  call  startCritical  to start such a code region and
@@ -1227,30 +1226,14 @@ typedef struct functor_table
 
 #define FUNCTOR_IS_VALID(flags)		((flags) & VALID_F)
 
-
-#ifdef O_LOGICAL_UPDATE
-#define VISIBLE_CLAUSE(cl, gen) \
-	( ( (cl)->generation.created <= (gen) && \
-	    (cl)->generation.erased   > (gen) && \
-	    (cl)->generation.erased  != LD->gen_reload \
-	  ) || \
-	  ( (cl)->generation.created == LD->gen_reload \
-	  ) \
-	)
 #define GLOBALLY_VISIBLE_CLAUSE(cl, gen) \
 	( (cl)->generation.created <= (gen) && \
 	  (cl)->generation.erased   > (gen) \
 	)
-#else
-#define VISIBLE_CLAUSE(cl, gen) false(cl, CL_ERASED)
-#define GLOBALLY_VISIBLE_CLAUSE(cl, gen) false(cl, CL_ERASED)
-#endif
 
 #define visibleClause(cl, gen) visibleClause__LD(cl, gen PASS_LD)
 #define visibleClauseCNT(cl, gen) visibleClauseCNT__LD(cl, gen PASS_LD)
 
-#define GEN_INVALID 0
-
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Struct clause must be a  multiple   of  sizeof(word)  for compilation on
 behalf  of  I_USERCALL.  This   is   verified    in   an   assertion  in
@@ -1274,6 +1257,7 @@ struct clause
   unsigned int		source_no;	/* Index of source-file */
   unsigned int		owner_no;	/* Index of owning source-file */
   unsigned int		references;	/* # ClauseRef pointing at me */
+  unsigned int		tr_erased_no;	/* # transactions that erased me */
   code			code_size;	/* size of ->codes */
   code			codes[1];	/* VM codes of clause */
 };
