@@ -50,6 +50,9 @@
   #endif
 #endif
 
+#include "pl-transaction.h"
+
+
 		 /*******************************
 		 *	 LOCK-FREE SUPPORT	*
 		 *******************************/
@@ -471,7 +474,25 @@ register_attvar(Word gp ARG_LD)
 
 static inline int
 visibleClause__LD(Clause cl, gen_t gen ARG_LD)
-{ return VISIBLE_CLAUSE(cl, gen);
+{ gen_t c, e;
+
+  c = cl->generation.created;
+  MEMORY_ACQUIRE();
+  e = cl->generation.erased;
+
+  if ( unlikely(e == LD->gen_reload) )
+    return FALSE;
+  if ( unlikely(c == LD->gen_reload) )
+    return TRUE;
+
+  if ( c <= gen && e > gen )
+    return TRUE;
+
+  if ( unlikely(gen >= LD->transaction.gen_base) &&
+       true(cl->predicate, P_DYNAMIC) )
+    return transaction_visible_clause(cl, gen PASS_LD);
+
+  return FALSE;
 }
 
 static inline int
@@ -482,52 +503,44 @@ visibleClauseCNT__LD(Clause cl, gen_t gen ARG_LD)
   return FALSE;
 }
 
-#ifdef ATOMIC_GENERATION_HACK
-/* Work around lacking 64-bit atomic operations.  These are designed to
-   be safe if we assume that read and increment complete before other
-   threads incremented 4G generations.
-*/
-
-static inline gen_t
-global_generation(void)
-{ gen_t g;
-  gen_t last;
-
-  do
-  { last = GD->_last_generation;
-    g = (gen_t)GD->_generation.gen_u<<32 | GD->_generation.gen_l;
-  } while ( unlikely(g < last) );
-
-  if ( unlikely(last != g) )
-    GD->_last_generation = g;
-
-  return g;
-}
-
-static inline gen_t
-next_global_generation(void)
-{ uint32_t u = GD->_generation.gen_u;
-  uint32_t l;
-
-  if ( unlikely((l=ATOMIC_INC(&GD->_generation.gen_l)) == 0) )
-    u = ATOMIC_INC(&GD->_generation.gen_u);
-
-  return (gen_t)u<<32|l;
-}
-
-#else /*ATOMIC_GENERATION_HACK*/
-
 static inline gen_t
 global_generation(void)
 { return GD->_generation;
 }
 
 static inline gen_t
-next_global_generation(void)
-{ return ATOMIC_INC(&GD->_generation);
+current_generation(Definition def ARG_LD)
+{ if ( unlikely(!!LD->transaction.generation) && def && true(def, P_DYNAMIC) )
+  { return LD->transaction.generation;
+  } else
+  { return GD->_generation;
+  }
 }
 
-#endif /*ATOMIC_GENERATION_HACK*/
+static inline gen_t
+next_generation(Definition def ARG_LD)
+{ if ( unlikely(!!LD->transaction.generation) && def && true(def, P_DYNAMIC) )
+  { if ( LD->transaction.generation < LD->transaction.gen_max )
+      return ++LD->transaction.generation;
+    return 0;
+  } else
+  { gen_t gen;
+
+    PL_LOCK(L_GENERATION);
+    gen = ++GD->_generation;
+    PL_UNLOCK(L_GENERATION);
+
+    return gen;
+  }
+}
+
+static inline gen_t
+max_generation(Definition def ARG_LD)
+{ if ( unlikely(!!LD->transaction.generation) && def && true(def, P_DYNAMIC) )
+    return LD->transaction.gen_max;
+  else
+    return GEN_MAX;
+}
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 We must ensure that cleanDefinition() does   not remove clauses that are
@@ -542,15 +555,17 @@ generation is updated and thus no harm is done.
 
 static inline void
 setGenerationFrame__LD(LocalFrame fr ARG_LD)
-{
-#ifdef O_LOGICAL_UPDATE
-  gen_t gen;
+{ if ( unlikely(LD->transaction.generation &&
+		true(fr->predicate, P_DYNAMIC)) )
+  { setGenerationFrameVal(fr, LD->transaction.generation);
+  } else
+  { gen_t gen;
 
-  do
-  { gen = global_generation();
-    setGenerationFrameVal(fr, gen);
-  } while(gen != global_generation());
-#endif
+    do
+    { gen = global_generation();
+      setGenerationFrameVal(fr, gen);
+    } while(gen != global_generation());
+  }
 }
 
 static inline int
@@ -609,34 +624,26 @@ normally 0). longToPointer() does the inverse operation.
 
 static inline uintptr_t
 pointerToInt(void *ptr)
-{
-#if SIZEOF_VOIDP == 8
-  return (uintptr_t) ptr;
-#else
-  uintptr_t p   = (uintptr_t) ptr;
+{ uintptr_t p   = (uintptr_t) ptr;
   uintptr_t low = p & 0x3L;
 
+  p -= GD->heap_base;
   p >>= 2;
   p |= low<<(sizeof(uintptr_t)*8-2);
 
   return p;
-#endif
 }
 
 
 static inline void *
 intToPointer(uintptr_t p)
-{
-#if SIZEOF_VOIDP == 8
-  return (void*) p;
-#else
-  uintptr_t low = p >> (sizeof(uintptr_t)*8-2);
+{ uintptr_t low = p >> (sizeof(uintptr_t)*8-2);
 
   p <<= 2;
   p |= low;
+  p += GD->heap_base;
 
   return (void *) p;
-#endif
 }
 
 #endif /*PL_INLINE_H_INCLUDED*/
