@@ -223,9 +223,10 @@ extern counting_mutex _PL_mutexes[];	/* Prolog mutexes */
 #define L_EVHOOK       26
 #define L_OSDIR	       27
 #define L_ALERT	       28
+#define L_GENERATION   29
 #ifdef __WINDOWS__
-#define L_DDE	       29
-#define L_CSTACK       30
+#define L_DDE	       30
+#define L_CSTACK       31
 #endif
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -452,8 +453,8 @@ COMMON(void)		free_wait_area(thread_wait_area *wa);
 		 *******************************/
 
 COMMON(void)	forThreadLocalDataUnsuspended(
-		    void (*func)(struct PL_local_data *),
-		    unsigned flags);
+		    void (*func)(struct PL_local_data *, void *ctx),
+		    void *ctx);
 COMMON(void)	resumeThreads(void);
 COMMON(void)	markAtomsMessageQueues(void);
 COMMON(void)	markAtomsThreadMessageQueue(PL_local_data_t *ld);
